class oci::compute(
  $machine_hostname         = undef,
  $machine_ip               = undef,
  $all_masters              = undef,
  $all_masters_ip           = undef,
  $vip_hostname             = undef,
  $vip_ipaddr               = undef,
  $vip_netmask              = undef,
  $network_ipaddr           = undef,
  $network_cidr             = undef,
  $use_ssl                  = true,
  $pass_nova_messaging      = undef,
  $pass_nova_authtoken      = undef,
  $pass_neutron_authtoken   = undef,
  $pass_metadata_proxy_shared_secret = undef,
  $pass_neutron_messaging   = undef,
  $pass_placement_authtoken = undef,

  $cluster_has_osds         = undef,
  $ceph_fsid                = undef,
  $ceph_libvirtuuid         = undef,
  $ceph_bootstrap_osd_key   = undef,
  $ceph_admin_key           = undef,
  $ceph_openstack_key       = undef,
  $ceph_mon_key             = undef,
  $ceph_mon_initial_members = undef,
  $ceph_mon_host            = undef,

){

  if $use_ssl {
    $proto = 'https'
    $messaging_default_port = '5671'
    $messaging_notify_port = '5671'
    $api_port = 443
  } else {
    $proto = 'http'
    $messaging_default_port = '5672'
    $messaging_notify_port = '5672'
    $api_port = 80
  }
  $messaging_default_proto = 'rabbit'
  $messaging_notify_proto = 'rabbit'

  $base_url = "${proto}://${vip_hostname}"
  $keystone_auth_uri  = "${base_url}:${api_port}/identity"
  $keystone_admin_uri = "${base_url}:${api_port}/identity-admin"

  # We need haproxy for proxying the metadata proxy server
  # because of TLS + Eventlet + Python 3
  class { 'haproxy':
    global_options   => {
      'maxconn' => '256',
      'user'    => 'haproxy',
      'group'   => 'haproxy',
      'daemon'  => '',
    },
    defaults_options => {
      'mode'    => 'http',
    },
    merge_options => true,
  }

  $haproxy_options_for_metadata_proxy = [
    { 'use_backend' => 'metadatabe' },
  ]
  haproxy::frontend { 'openstackfe':
    mode      => 'http',
    bind      => { "127.0.0.1:8775" => [] },
    options   => $haproxy_options_for_metadata_proxy,
  }
  haproxy::backend { 'metadatabe':
    options => [
       { 'option' => 'forwardfor' },
       { 'mode' => 'http' },
       { 'balance' => 'roundrobin' },
    ],
  }
  haproxy::balancermember { 'metadatabm':
    listening_service => 'metadatabe',
    ipaddresses       => $all_masters_ip,
    server_names      => $all_masters,
    ports             => 8775,
#    options           => 'check',
  }

  ############
  ### Nova ###
  ############
  class { '::nova':
    default_transport_url      => os_transport_url({
      'transport' => $messaging_default_proto,
      'hosts'     => $all_masters,
      'port'      => $messaging_default_port,
      'username'  => 'nova',
      'password'  => $pass_nova_messaging,
    }),
    notification_transport_url => os_transport_url({
      'transport' => $messaging_notify_proto,
      'hosts'     => $all_masters,
      'port'      => $messaging_notify_port,
      'username'  => 'nova',
      'password'  => $pass_nova_messaging,
    }),
    rabbit_use_ssl                => $use_ssl,
    amqp_sasl_mechanisms          => 'PLAIN',
    use_ipv6                      => false,
    glance_api_servers            => "${base_url}/image",
    notification_driver           => 'messagingv2',
    notify_on_state_change        => 'vm_and_task_state',
  }

  class { '::nova::keystone::authtoken':
    password             => $pass_nova_authtoken,
    auth_url             => $keystone_admin_uri,
    www_authenticate_uri => $keystone_auth_uri,
    cafile               => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
  }

  nova_config {
    'placement/cafile':       value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
    'placement/auth_url':     value => $keystone_admin_uri;
    'placement/region_name':  value => 'RegionOne';
    'placement/password':     value => $pass_placement_authtoken;
    'placement/project_name': value => 'services';
    'glance/cafile':          value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
    'keystone/cafile':        value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
    'neutron/cafile':         value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
    'neutron/endpoint_override': value => "${base_url}/network";
  }

  # Needed, so puppet-openstack can restart virtlogd and libvirtd after configuration
  # so it listen to TCP before nova-compute starts.
  include ::nova::compute::libvirt::services
  Service['virtlogd'] -> Service['libvirtd']

  class { '::nova::compute':
    vnc_enabled                 => true,
    instance_usage_audit        => true,
    instance_usage_audit_period => 'hour',
    keymgr_api_class            => 'castellan.key_manager.barbican_key_manager.BarbicanKeyManager',
    barbican_auth_endpoint      => "${base_url}/identity/v3",
    barbican_endpoint           => "${base_url}/keymanager",
  }

  class { '::nova::compute::libvirt':
    libvirt_virt_type       => 'kvm',
    libvirt_cpu_mode        => 'host-passthrough',
    # Setting-up cpu_model doesn't seem to work in the KVM PoC.
    # This is to be investigated.
#    libvirt_cpu_mode        => 'custom',
#    libvirt_cpu_model       => 'SandyBridge-IBRS',
    migration_support       => true,
    # False is needed because of the above include ::nova::compute::libvirt::services
    manage_libvirt_services => false,
  }

  class { '::nova::network::neutron':
    neutron_auth_url      => "${base_url}/identity-admin/v3",
    neutron_url           => "${base_url}/network",
    neutron_password      => $pass_neutron_authtoken,
    default_floating_pool => 'public',
  }

  if $cluster_has_osds {
    class { 'ceph':
      fsid                => $ceph_fsid,
      ensure              => 'present',
      authentication_type => 'cephx',
      mon_initial_members => $ceph_mon_initial_members,
      mon_host            => $ceph_mon_host,
    }->
    ceph::key { 'client.admin':
      secret  => $ceph_admin_key,
      cap_mon => 'allow *',
      cap_osd => 'allow *',
      cap_mds => 'allow',
    }->
    ceph::key { 'client.openstack':
      secret  => $ceph_openstack_key,
      mode    => '0644',
      cap_mon => 'profile rbd',
      cap_osd => 'profile rbd pool=cinder, profile rbd pool=nova, profile rbd pool=glance, profile rbd pool=gnocchi',
    }
    class { '::nova::compute::rbd':
      libvirt_rbd_user        => 'openstack',
      libvirt_rbd_secret_uuid => $ceph_libvirtuuid,
      libvirt_rbd_secret_key  => $ceph_openstack_key,
      libvirt_images_rbd_pool => 'nova',
      rbd_keyring             => 'client.openstack',
      # ceph packaging is already managed by puppet-ceph
      manage_ceph_client      => false,
      require => Ceph::Key['client.openstack'],
    }
    # make sure ceph pool exists before running nova-compute
    # Note: cannot be done on the compute nodes, since we aren't doing pools here
    # but on the controller.
    #Exec['create-nova'] -> Service['nova-compute']
  }

  ###############
  ### Neutron ###
  ###############
  class { '::neutron':
    default_transport_url      => os_transport_url({
      'transport' => $messaging_default_proto,
      'hosts'     => $all_masters,
      'port'      => $messaging_default_port,
      'username'  => 'neutron',
      'password'  => $pass_neutron_messaging,
    }),
    notification_transport_url => os_transport_url({
      'transport' => $messaging_notify_proto,
      'hosts'     => $all_masters,
      'port'      => $messaging_notify_port,
      'username'  => 'neutron',
      'password'  => $pass_neutron_messaging,
    }),
    rabbit_use_ssl             => $use_ssl,
    amqp_sasl_mechanisms       => 'PLAIN',
    allow_overlapping_ips      => true,
    core_plugin                => 'ml2',
    service_plugins            => ['router', 'metering', 'firewall', 'qos', 'trunk',],
    debug                      => true,
    bind_host                  => $machine_ip,
    notification_driver        => 'messagingv2',
  }
  neutron_config {
    'nova/cafile':   value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
  }

  class { '::neutron::server::notifications':
    auth_url => $keystone_admin_uri,
    password => $pass_nova_authtoken,
  }

  class { '::neutron::client': }
  class { '::neutron::keystone::authtoken':
    password             => $pass_neutron_authtoken,
    auth_url             => $keystone_admin_uri,
    www_authenticate_uri => $keystone_auth_uri,
    cafile               => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
  }
  class { '::neutron::plugins::ml2':
    type_drivers         => ['vxlan', 'vlan', 'flat'],
    tenant_network_types => ['vxlan', 'vlan', 'flat'],
    extension_drivers    => 'port_security,qos',
    mechanism_drivers    => 'openvswitch',
    firewall_driver      => 'iptables_hybrid',
  }
  class { '::neutron::agents::l3':
    interface_driver => 'openvswitch',
    debug            => true,
    extensions       => 'fwaas',
  }
  class { '::neutron::agents::dhcp':
    interface_driver => 'openvswitch',
    debug            => true,
  }
  class { '::neutron::agents::metering':
    interface_driver => 'openvswitch',
    debug            => true,
  }
  class { '::neutron::services::fwaas':
    enabled       => true,
    agent_version => 'v1',
    driver        => 'neutron_fwaas.services.firewall.drivers.linux.iptables_fwaas.IptablesFwaasDriver',
  }
  class { '::neutron::agents::metadata':
    debug             => true,
    shared_secret     => $pass_metadata_proxy_shared_secret,
    metadata_workers  => 2,
    package_ensure    => 'latest',
    metadata_protocol => 'http',
    metadata_host     => "127.0.0.1",
#    metadata_insecure => true,
  }
  class { '::neutron::agents::ml2::ovs':
    local_ip   => $machine_ip,
    tunnel_types    => ['vxlan'],
    bridge_mappings => ['external:br-ex'],
    extensions => 'fwaas',
  }
}
